"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.login = exports.loginClient = exports.pwHash = exports.logout = exports.reqReadEntities = exports.reqEntityConf = exports.presenceFor = exports.fromChannel = exports.req = exports.unsubEntityUpdates = exports.subEntityUpdates = exports.ridTobusChannel = exports.unsub = exports.sub = exports.pub = void 0;

var _extendedDsClient = _interopRequireWildcard(require("extended-ds-client"));

var _sha = _interopRequireDefault(require("crypto-js/sha256"));

var R = _interopRequireWildcard(require("ramda"));

var _jstiip = _interopRequireDefault(require("jstiip"));

var _subSeconds = _interopRequireDefault(require("date-fns/fp/subSeconds"));

var _get = _interopRequireDefault(require("lodash/fp/get"));

var _dmUtils = require("./dm-utils");

var _transforms = require("./transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { if (i % 2) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } else { Object.defineProperties(target, Object.getOwnPropertyDescriptors(arguments[i])); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Publishes a message through the backend.
 * @param {string} channel - The channel to publish on.
 * @param {Array} [pl=[]] - The payload to send.
 * @param {string} [sig] - The signal data to apply to the published message.
 */
var pub = function pub(channel) {
  var pl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var sig = arguments.length > 2 ? arguments[2] : undefined;
  var msg = new _jstiip["default"]({
    pl: pl
  });
  if (sig) msg.sig = sig;
  var msgOut = msg.toJS();

  _extendedDsClient.ds.client.event.emit(channel, msgOut); // console.log('PUB:', channel, msgOut);

};
/**
 * Subscribes to a channel on the backend.
 * @param  {function} callback - The callback to trigger when a message arrives.
 * @param  {string} channel - The channel to subscribe to.
 */


exports.pub = pub;
var sub = R.curry(function (callback, channel) {
  return _extendedDsClient.ds.client.event.subscribe(channel, callback);
});
/**
 * Unsubscribes from a channel on the backend.
 * @param  {string} channel - The channel to unsubscribe from.
 * @param  {function} [callback] - The callback that was previously registered with subscribe. If
 *  the callback is omitted, all subscriptions for the channel will be removed.
 */

exports.sub = sub;

var unsub = function unsub(channel, callback) {
  return _extendedDsClient.ds.client.event.unsubscribe(channel, callback);
};

exports.unsub = unsub;
var confUpdatesChannel = R.curry(function (type, rid) {
  return "conf/".concat(type, "/update/").concat(rid);
});

var ridTobusChannel = function ridTobusChannel(rid) {
  return "data/".concat(rid);
};

exports.ridTobusChannel = ridTobusChannel;

var subEntityUpdates = function subEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (cb) {
    return R.forEach(R.o(sub(cb), ridToChannel));
  };
};

exports.subEntityUpdates = subEntityUpdates;

var unsubEntityUpdates = function unsubEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (rids, cb) {
    return R.forEach(R.o(function (ch) {
      return unsub(ch, cb);
    }, ridToChannel))(rids);
  };
};
/**
 * Performs a request to the backend.
 * @param  {string} id - The id of the rpc
 * @param  {Object} [args={}] - Arguments passed on to the rpc
 * @return {any} - Depends on the rpc.
 */
// TODO: Refactor temporarily to rids:* to target all, but only for 'conf...'-requests


exports.unsubEntityUpdates = unsubEntityUpdates;
var req = R.curry(function (id, args) {
  // console.log('req:', id, args);
  return _extendedDsClient.ds.client.rpc.p.make(id, args)["catch"](function (err) {
    console.error(id, args, err);
    throw err;
  });
});
exports.req = req;

var fromChannel = function fromChannel(_ref) {
  var rid = _ref.rid,
      last = _ref.last,
      tail = _ref.tail,
      ttl = _ref.ttl,
      cb = _ref.cb,
      noSub = _ref.noSub,
      path = _ref.path,
      noTsToDate = _ref.noTsToDate,
      projection = _ref.projection,
      busChannel = _ref.busChannel;
  if (rid === undefined && busChannel === undefined) return undefined;
  var callback = R.compose(cb, R.when(function () {
    return path !== undefined;
  }, (0, _get["default"])(path)), R.when(function () {
    return !noTsToDate;
  }, _dmUtils.evolveTsToDate), _dmUtils.ensureTsV3);

  if (last || tail) {
    req('data-history/read', _objectSpread({
      channel: rid
    }, ttl ? {
      from: (0, _dmUtils.dateToTs)((0, _subSeconds["default"])(ttl)(new Date()))
    } : {}, {
      limit: tail || Number(last),
      projection: projection
    })).then(function (reply) {
      return reply.reverse().forEach(callback);
    });
  }

  if (!noSub) {
    var channel = busChannel || ridTobusChannel(rid);
    sub(callback, channel);

    var unsubscribe = function unsubscribe() {
      return unsub(channel, callback);
    };

    return unsubscribe; // notify caller about a subscription made
  }

  return undefined;
};

exports.fromChannel = fromChannel;

var presenceFor = function presenceFor(_ref2) {
  var id = _ref2.id,
      cb = _ref2.cb,
      noSub = _ref2.noSub;
  if (id === undefined) return undefined;
  req('presence/listClients', {}).then(function (reply) {
    return cb(reply.includes(id));
  });

  if (!noSub) {
    var channel = "presence/".concat(id);
    sub(cb, channel);

    var unsubscribe = function unsubscribe() {
      return unsub(channel, cb);
    };

    return unsubscribe; // notify caller about a subscription made
  }

  return undefined;
};

exports.presenceFor = presenceFor;
var reqEntityConf = R.curry(function (id, entityRid) {
  return req('conf/readConfiguration', _objectSpread({
    entities: (0, _transforms.ensureArray)(entityRid),
    rids: []
  }, id ? {
    filter: ['=', 'id', id]
  } : {})).then(function (results) {
    return id && typeof entityRid === 'string' ? results[0] : results;
  });
});
exports.reqEntityConf = reqEntityConf;

var entitiesConfigsMapper = function entitiesConfigsMapper(configIds) {
  return R.compose(R.chain(function (entityRid) {
    return R.map(function (id) {
      return {
        id: id,
        entityRid: entityRid
      };
    })(configIds);
  }), R.pluck('rid'));
};

var inflateConfMap = R.map(function (obj) {
  return reqEntityConf(obj.id, obj.entityRid).then(R.compose( // R.tap(console.log),
  R.mergeLeft(obj), R.objOf('config')));
});

var injectConfsInto = function injectConfsInto(entities) {
  return R.compose(R.values, R.reduce(function (enties, _ref3) {
    var config = _ref3.config,
        id = _ref3.id,
        entityRid = _ref3.entityRid;
    if (config && R.has(entityRid)(enties)) return R.evolve(_defineProperty({}, entityRid, (0, _transforms.ensureAndEvolveKey)({}, 'configurations', R.mergeLeft(_defineProperty({}, id, config)))))(enties);
    return enties;
  }, (0, _dmUtils.formatRecords)(entities)));
};
/**
 * reqReadEntities - Abstracted req call with support for configurationsToInclude from future.
 *
 * @param {object} args The arguments for the call. Supports configurationsToInclude.
 *
 * @returns {object} Collection (list) of entities read, or one entity for specific requests.
 */


var reqReadEntities =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(rpc, args) {
    var _args$configurationsT, configurationsToInclude, rest, reply, entities, confsToRead, confRequests, configsWithRefs, result;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // TODO: when configuration duck is in place, only config rids should be put in each asset's
            // 'configurations', and configs should then be fetched separately into the configuration duck
            _args$configurationsT = args.configurationsToInclude, configurationsToInclude = _args$configurationsT === void 0 ? [] : _args$configurationsT, rest = _objectWithoutProperties(args, ["configurationsToInclude"]);
            _context.next = 3;
            return req(rpc, rest);

          case 3:
            reply = _context.sent;
            entities = (0, _transforms.ensureArray)(reply);
            confsToRead = entitiesConfigsMapper(configurationsToInclude)(entities); // console.log('confsToRead:', confsToRead);

            confRequests = inflateConfMap(confsToRead);
            _context.next = 9;
            return Promise.all(confRequests);

          case 9:
            configsWithRefs = _context.sent;
            // console.log('configsWithRefs (should be list):', configsWithRefs);
            result = injectConfsInto(entities)(configsWithRefs); // console.log('result:', result);

            return _context.abrupt("return", Array.isArray(reply) ? result : R.prop(0, result));

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function reqReadEntities(_x, _x2) {
    return _ref4.apply(this, arguments);
  };
}(); // /**
//  * Converts a rid to be url-friendly
//  * @param {string} rid - The record id to convert.
//  * @return {string} - An url-friendly rid.
//  */
// export const ridToUrl = R.compose(
//   R.replace(':', '-'),
//   R.replace('#', ''),
// );
//
// /**
//  * Converts a url-friendly rid to an actual rid.
//  * @param {string} rid - The url-friendly record id to convert.
//  * @return {string} - An actual record id.
//  */
// export const urlToRid = R.compose(
//   rid => `#${rid}`,
//   R.replace('-', ':'),
// );


exports.reqReadEntities = reqReadEntities;

var logout = function logout() {
  if (!_extendedDsClient.ds.client) return;

  _extendedDsClient.ds.client.off();

  _extendedDsClient.ds.client.close();
};
/**
 * Creates a connection to the backend server.
 * @param {string} serverAddress
 * @param {function} onConnectionError - Callback triggered on connection errors.
 * @param {function} onUnexpectedError - Callback triggered on general errors in the client.
 * @param {}
 * @return {object} - An extended deepstream client
 */


exports.logout = logout;

var createConnection = function createConnection(serverAddress, onUnexpectedError) {
  console.log('createConnection:', serverAddress);
  var client = (0, _extendedDsClient["default"])(serverAddress);
  client.on('error', onUnexpectedError);
  return client;
};

var pwHash = function pwHash(pw) {
  return String((0, _sha["default"])(pw));
};

exports.pwHash = pwHash;

var loginClient =
/*#__PURE__*/
function () {
  var _ref5 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(client, serverAddress, id, password, onConnectionError) {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return client.p.login({
              id: id,
              password: password
            });

          case 2:
            client.on('connectionStateChanged', function (state) {
              // We need to set this only when we know that we got a working logged in connection
              console.log('CONNECTION STATE:', state);
              if (state === 'ERROR' && client.connectionState !== 'ERROR') onConnectionError();
              /* eslint-disable no-param-reassign */

              client.connectionState = state;
              /* eslint-enable no-param-reassign */
            });
            return _context2.abrupt("return", {
              id: id,
              password: password,
              serverAddress: serverAddress
            });

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function loginClient(_x3, _x4, _x5, _x6, _x7) {
    return _ref5.apply(this, arguments);
  };
}();

exports.loginClient = loginClient;

var login =
/*#__PURE__*/
function () {
  var _ref6 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(serverAddress, id, password, onConnectionError, onUnexpectedError) {
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (_extendedDsClient.ds.client) logout();
            _extendedDsClient.ds.client = createConnection(serverAddress, onUnexpectedError);
            return _context3.abrupt("return", loginClient(_extendedDsClient.ds.client, serverAddress, id, password, onConnectionError));

          case 3:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function login(_x8, _x9, _x10, _x11, _x12) {
    return _ref6.apply(this, arguments);
  };
}(); // const ensureRids = ensureKey([], 'rids');
//
// export const fetchEntity = type => {
//   if (!fetchMapping[type]) throw new Error('No such entity type:', type);
//   return async (args = {}) => {
//     let records = [];
//     try {
//       records = await req(`conf/${fetchMapping[type]}`, ensureRids(args));
//     } catch (err) {
//       console.warn(`Fetching ${type}s failed:`, err);
//     }
//     return R.indexBy(R.prop('rid'), records);
//   };
// };


exports.login = login;