"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.busChannel = exports.confUpdatesChannel = exports.evolveTsToDate = exports.tsToPl = exports.tsToDate = exports.dateToTs = exports.formatRecords = exports.ensureRidsInArgs = exports.ensureTsV3 = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _dataUtils = require("./data-utils");

var _transforms = require("./transforms");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { if (i % 2) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } else { Object.defineProperties(target, Object.getOwnPropertyDescriptors(arguments[i])); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ensureTsV3 = function ensureTsV3(msg) {
  var ct = msg.ct;

  if (ct !== undefined) {
    if (Object.prototype.toString.call(ct) === '[object Date]') return _objectSpread({}, msg, {
      ts: ct
    });
    return _objectSpread({}, msg, {
      ts: new Date(Number(ct) * 1000).toISOString()
    });
  } // if (Object.prototype.toString.call(ts) === '[object Date]') return msg;


  return msg;
};

exports.ensureTsV3 = ensureTsV3;
var ensureRidsInArgs = R.compose((0, _transforms.ensureKey)([], 'rids'), R.when(R.isNil, R.always({})));
exports.ensureRidsInArgs = ensureRidsInArgs;
var formatRecords = R.indexBy(R.prop('rid'));
exports.formatRecords = formatRecords;

var dateToTs = function dateToTs(da) {
  return da.toISOString();
};

exports.dateToTs = dateToTs;

var tsToDate = function tsToDate(ts) {
  return (0, _dataUtils.utcDateToDate)(new Date(ts));
};

exports.tsToDate = tsToDate;

var tsToPl = function tsToPl(msg) {
  return R.prepend(msg.ts, msg.pl);
};

exports.tsToPl = tsToPl;
var evolveTsToDate = R.evolve({
  ts: tsToDate
});
exports.evolveTsToDate = evolveTsToDate;
var confUpdatesChannel = R.curry(function (type, rid) {
  return "conf/".concat(type, "/update/").concat(rid);
});
exports.confUpdatesChannel = confUpdatesChannel;

var busChannel = function busChannel(rid) {
  return "data/".concat(rid);
};

exports.busChannel = busChannel;